"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
const defaultMaxNumberSuggestions = 10;
const baseCost = 100;
const swapCost = 75;
const postSwapCost = swapCost - baseCost;
const maxNumChanges = 5;
function suggest(root, word, maxNumSuggestions = defaultMaxNumberSuggestions) {
    const collector = suggestionCollector(word, maxNumSuggestions);
    genSuggestions(root, word, collector);
    return collector.suggestions;
}
exports.suggest = suggest;
function genSuggestions(root, word, output) {
    const bc = baseCost;
    const psc = postSwapCost;
    const matrix = [[]];
    const x = ' ' + word;
    const mx = x.length - 1;
    let costLimit = Math.min(bc * word.length / 2, bc * maxNumChanges);
    for (let i = 0; i <= mx; ++i) {
        matrix[0][i] = i * baseCost;
    }
    const i = util_1.walker(root);
    let deeper = true;
    for (let r = i.next(deeper); !r.done; r = i.next(deeper)) {
        const { text, node, depth } = r.value;
        const d = depth + 1;
        const lastSugLetter = d > 1 ? text[d - 2] : '';
        const w = text.slice(-1);
        matrix[d] = matrix[d] || [];
        matrix[d][0] = matrix[d - 1][0] + bc;
        let lastLetter = x[0];
        let min = matrix[d][0];
        for (let i = 1; i <= mx; ++i) {
            let curLetter = x[i];
            const c = bc - d;
            let subCost = (w === curLetter)
                ? 0
                : (curLetter === lastSugLetter ? (w === lastLetter ? psc : c) : c);
            matrix[d][i] = Math.min(matrix[d - 1][i - 1] + subCost, // substitute
            matrix[d - 1][i] + c, // insert
            matrix[d][i - 1] + c // delete
            );
            min = Math.min(min, matrix[d][i]);
            lastLetter = curLetter;
        }
        let cost = matrix[d][mx];
        if (util_1.isWordTerminationNode(node) && cost <= costLimit) {
            costLimit = output({ word: text, cost });
        }
        deeper = (min <= costLimit);
    }
}
exports.genSuggestions = genSuggestions;
function suggestionCollector(word, maxNumSuggestions, filter = () => true) {
    const sugs = [];
    let maxCost = Math.min(baseCost * word.length / 2, baseCost * maxNumChanges);
    ;
    function comp(a, b) {
        return a.cost - b.cost || a.word.length - b.word.length || a.word.localeCompare(b.word);
    }
    function collector(suggestion) {
        if (suggestion.cost <= maxCost && filter(suggestion.word)) {
            sugs.push(suggestion);
            if (sugs.length > maxNumSuggestions) {
                sugs.sort(comp);
                sugs.length = maxNumSuggestions;
                maxCost = sugs[sugs.length - 1].cost;
            }
        }
        return maxCost;
    }
    const sugCollector = collector;
    Object.defineProperties(sugCollector, {
        suggestions: { get: () => sugs.sort(comp) },
        maxCost: { get: () => maxCost },
        word: { get: () => word },
    });
    return sugCollector;
}
exports.suggestionCollector = suggestionCollector;
//# sourceMappingURL=suggest.js.map